#include <gtk/gtk.h>
#include <glib/gprintf.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

void saveHexs(GdkPixbuf *pb)
{
    // Map is 4000x2000 pixels 200 hexs across. 86.5 hexes down
    int row=78, col=6; // Row and column of the target hex
    int sTarget=0; // sTarget is starting location in source for current hex

    int sh, sw; // Source is 4000 wide x 2000 high
    int dh, dw; // Destination is 20 wide x 20 high
    int x,y,i; // Declare ints for loop
    int bpp=3; // Bits per pixel
    int sRowstride, dRowstride; // Width in bytes of each row
    gchar *savefile; // Save file name
    // Declare Widget for single hex map. This will be reused
    GtkWidget *mapImage;
    // The array containing the pixel buffer data
    GdkPixbuf *pixel;
    guchar *sPixels, *dPixels;

    // Get the source image width
    sw=gdk_pixbuf_get_width(pb); //4000
    sh=gdk_pixbuf_get_height(pb); //2000

    // Create Widget and load the single hex picture into it
    mapImage = gtk_image_new_from_file("/home/tal/Pictures/one_hex.png");

    // Get pointer to pixels.
    pixel=gtk_image_get_pixbuf(mapImage);
    if(gdk_pixbuf_get_bits_per_sample(pixel)!=8) return;
    if(gdk_pixbuf_get_bits_per_sample(pb)!=8) return;
    dw=gdk_pixbuf_get_width(pixel); //20
    dh=gdk_pixbuf_get_height(pixel); //20

    sRowstride=gdk_pixbuf_get_rowstride(pb);//sw*bpp; // Source
    dRowstride=gdk_pixbuf_get_rowstride(pixel);//dw*bpp; // Destination

    // The pixel buffer in the source GdkPixbuf instance
    sPixels = gdk_pixbuf_get_pixels(pb);
    // The pixel buffer in the destination GdkPixbuf instance
    dPixels = gdk_pixbuf_get_pixels(pixel);


    for(row=0; row<(sh/dh); row++)
    {
        for(col=0; col<(sw/dw); col++)
        {
            sTarget=(sRowstride*row*dh)+(col*dRowstride); //bpp*dw);

            // Copy pixels over
            for(y=0; y<dh; y++) //iterate over the height of image.
            {
                for(x=0; x<dRowstride; x+=bpp) //read every pixel in the row.skip bpp bytes
                {
                  // i iterates through r,g,b
                    for(i=0; i<bpp; i++)
                    {
                        dPixels[(y*dRowstride) +x+i]=sPixels[sTarget+(y*sRowstride) + x+i];
                    }
                }
            }
            // Done copying. Save.
            savefile=g_strdup_printf("/home/tal/Pictures/map/map_%03d_%03d_%s",row,col,".png"); // Create a file name. The %02d pads with zeroes.
            gdk_pixbuf_save(gtk_image_get_pixbuf(mapImage),savefile,"png",NULL,NULL);//save the file
        }
    }
}

int main (int argc, char *argv[])
{
    // Initialize the GTK+ and all of its supporting libraries.
    gtk_init(&argc, &argv);
    // Declare Widget for my window
    GtkWidget *window;
    // Declare Widget for map
    GtkWidget *mapImage;
    // Declare a scrollable container to hold the map image
    GtkWidget *scrolled_window;

    // Create a new window
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    // Add the title
    gtk_window_set_title(GTK_WINDOW (window), "Guladreg");
    // Tell gtk to quit when the close window is clicked
    g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);

    // Create Widget and load an image into it
    mapImage = gtk_image_new_from_file("/home/tal/Pictures/Guladreg.png");

    // Create my scrolling container
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);

    // Add my map object to the scrolling window
    gtk_scrolled_window_add_with_viewport(GTK_CONTAINER (scrolled_window), mapImage);

    // Add it to the window
    gtk_container_add(GTK_CONTAINER (window), scrolled_window);

    // Set the window starting size
    gtk_widget_set_size_request(scrolled_window, 500, 500);

    // Show the window and all child containers
    gtk_widget_show_all(window);

    saveHexs(gtk_image_get_pixbuf(mapImage));

    // Hand control over to the main loop.
    gtk_main ();
    return 0;
}
